/////////////////////////////////////////////////////////////////////
//
// dsp56k.c
//
// handles communication to dsk board
//
// Date:		25.10.2009
// Author:		fcordes@cs.tu-berlin.de
//
/////////////////////////////////////////////////////////////////////

#ifndef _DSP56K_H_
#define _DSP56K_H_

#include "inttypes.h"
#include "hostport.h"
#include "dsp56k_reg.h"


#define DSPSYS_VID 0x03EB
#define DSPSYS_PID 0x1EE7



#define CMD_RP 0
#define CMD_WP 1
#define CMD_RX 2
#define CMD_WX 3
#define CMD_RY 4
#define CMD_WY 5

// access types used in dspsys messages

#define DSPSYS_READ_P 0x900
#define DSPSYS_READ_X 0x902
#define DSPSYS_READ_Y 0x904

#define DSPSYS_WRITE_P 0x901
#define DSPSYS_WRITE_X 0x903
#define DSPSYS_WRITE_Y 0x905

#define DSPSYS_READ_DIRECT 0x906
#define DSPSYS_WRITE_DIRECT 0x907

#define DSPSYS_RESET_ON 0x908
#define DSPSYS_RESET_OFF 0x909


#define USB_WRITE_EP 0x03	// Use EP3 for write commands
#define USB_READ_EP			// Use ??? for read commands

#define USB_MESSAGE_SIZE_MAX 64 // Assert FIFO to be 64 Byte
#define USB_PAYLOAD_SIZE_MAX USB_MESSAGE_SIZE_MAX - 4
#define USB_DATA_SIZE_MAX USB_PAYLOAD_SIZE_MAX - 4

// Addresses of the DSP ISR routines
#define ISR_RP 	0x13    		// Read P
#define ISR_WP	0x14			// Write P
#define ISR_RX	0x15			// Read X
#define ISR_WX	0x16			// Write X
#define ISR_RY	0x17			// Read Y
#define ISR_WY	0x18			// Write Y

#define ISR_HRD	0x20          // Host Receive Data Interrupt 
#define ISR_HTD	0x22          // Host Transmit Data Interrupt




#define INCREMENTAL_ADDRESSING (1<<0)



typedef struct host_message 
{
	uint32_t type;
	uint32_t addr;
	uint32_t count;
	uint32_t flags;
	uint32_t data[12];
} dspsys_host_message_t;


// Direct r/w to control lines
typedef struct DSPSYS_WRITE_INPUT_TAG
{
	uint32_t port; // Port
	uint8_t data; // Data
} DSPSYS_WRITE_INPUT;

/*
// r/w to dsp memory
typedef struct dspsys_indirect_access_payload
{
	uint16_t count;
	uint16_t addr;
	uint8_t data[USB_DATA_SIZE_MAX];
} dspsys_indirect_access_payload_t;

*/
typedef union inttype_convert
{
	uint8_t int8[2];
	uint16_t int16[2];
	uint32_t int32;
} inttype_convert_t;


typedef union {
	uint8_t readBuffer[64*sizeof(uint8_t)];
	dspsys_host_message_t msgBuffer;
} dspsys_message_buffer_t;


void dspsysInitIO ( void );

void dspsysCopyToFIFO (dspsys_host_message_t *pMsg);
dspsys_host_message_t* dspsysCopyFIFO (void);

void dspsysReadMemoryBlock(uint8_t target, uint32_t addr, uint32_t* data, uint32_t count, uint8_t flags);
void dspsysWriteMemoryBlock(uint8_t target, uint32_t addr, uint32_t* data, uint32_t count, uint8_t flags);
void dspsysReadRegister(uint8_t reg, uint8_t* data);
void dspsysWriteRegister(uint8_t reg, uint8_t* data);
uint8_t dspsysGetData (void);
void dspsysSetData(uint8_t data);
void dspsysSetAddr(uint8_t addr);

void dspsysDeactivateReset( void );
void dspsysActivateReset( void );
void dspsysDecodeMessage(dspsys_host_message_t *pMsg);


void dspsysDebugHostport(void);

#endif
