
#include "usart_ctl.h"
#include "usart_debug.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "avr/io.h"


volatile rwcmd_t cmdQueue[3];
volatile uint8_t queuePtr; 

void initQueue( void)
{
	/* Do not use Debug() in this function */
	//Debug("Setting up Serial Control...");
	rwcmd_t temp;
	temp.type=255;
	temp.addr=255;
	temp.data=255;
	cmdQueue[0]=temp;
	cmdQueue[1]=temp;
	cmdQueue[2]=temp;

	queuePtr = 0;


	//Debug("done!\r\n");
	//Debug("\r\nUSART-Control active\r\n\r\n");
	
}

void submitHostCommand(char* rxBuffer)
{
	rwcmd_t cmd;
	uint8_t addr=127;
	uint8_t data=127;
	char* substrPtr;
	uint8_t hasProcessed = 1;

	substrPtr = strtok(rxBuffer, ",");

	if ((substrPtr[0] == 'r') || (substrPtr[0] == 'R'))
	{
		// Read what?
		substrPtr = strtok(substrPtr, ",");
		if (substrPtr==0)
		{
			hasProcessed = 0;
		}
		addr = parseAddress(substrPtr);
		if (addr > 3)
		{
			hasProcessed = 0;
		}

		cmd.type = CMD_READ;
		cmd.addr = addr;
		cmd.data = data;
	} else if ((substrPtr[0] == 'w') || (substrPtr[0] == 'W'))
	{
		substrPtr = strtok(substrPtr, ",");
		if (substrPtr==0)
		{
			hasProcessed = 0;
		}
		addr = parseAddress(substrPtr);
		if (addr > 3)
		{
			hasProcessed = 0;
		}
		substrPtr = strtok(substrPtr, ",");
		if (substrPtr==0)
		{
			hasProcessed = 0;
		}
		data = parseData(substrPtr);

		cmd.type = CMD_WRITE;
		cmd.addr = addr;
		cmd.data = data;
	} else
	{
		hasProcessed = 0;
	}

	if (hasProcessed != 0)
	{
		queueCmd(cmd);
		Debug("QUEUED: ");
		dumpCmd(cmd);
	}
}
	

uint8_t parseAddress(const char* argPtr)
{
	uint8_t addr;
	if (strcmp(argPtr, "ICR")==0)
	{
		addr = 0;
	} else if (strcmp(argPtr, "CVR")==0)
	{
		addr = 1;
	} else if (strcmp(argPtr, "ISR")==0)
	{
		addr = 2;
	} else if (strcmp(argPtr, "IVR")==0)
	{
		addr = 3;
	} else
	{
		addr = 255;
	}
	return addr;
}


uint8_t parseData(const char* argPtr)
{
	return atoi(argPtr);
}


uint8_t queueCmd (rwcmd_t cmd)
{
	if (queuePtr <= 1)
	{
		cmdQueue[queuePtr++] = cmd;
	} else
	{
		return 0;
	}
	return queuePtr;
}


rwcmd_t dequeueCmd(uint8_t *result)
{
	rwcmd_t cmd;
	uint8_t i;
	if ((*result = queuePtr) > 0)
	{
		cmd = cmdQueue[0];	
		for (i = 1; i < queuePtr; i++)
		{
			cmdQueue[i-1] = cmdQueue[i];
		}
		queuePtr--;
	} else
	{
		cmd.addr = 0;
		cmd.type = 0;
		cmd.data = 0;
	}

	return cmd;
}	


void dumpCmd(rwcmd_t cmd)
{
	char str[80];

	sprintf(str, "%s to addr $%u [data: %u]\r\n", (cmd.type==CMD_READ)?"Read":"Write", cmd.addr, cmd.data);
	Debug(str);
}

